unit chartdemo;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TAMultiSeries, Forms, Controls,
  Graphics, Dialogs, ExtCtrls, StdCtrls, fpJSON, Jsonparser, fphttpclient;

type
  TDownloadCompleteEvent = procedure (AString: String) of object;

  TDownloadThread = class(TThread)
  private
    FPHTTPClient: TFPHTTPClient;
    FUrl: String;
    FOnDownloadComplete: TDownloadCompleteEvent;
    FString: String;
    procedure DoComplete;
  protected
    procedure Execute; override;
  public
    constructor Create(AUrl: String);
    destructor Destroy; override;
    property OnDownloadComplete: TDownloadCompleteEvent
      read FOnDownloadComplete write FOnDownloadComplete;
  end;


  { TForm1 }

  TForm1 = class(TForm)
    Chart1: TChart;
    currentmarket: TOpenHighLowCloseSeries;
    Label1: TLabel;
    Timer1: TTimer;
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    Procedure LoadGraf;
    procedure Timer1Timer(Sender: TObject);
  private
    FDownloadThread: TDownloadThread;
    procedure DownloadComplete(AJSONString: String);
    procedure ThreadTerminateHandler(Sender: TObject);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}


{ TDownloadThread }

constructor TDownloadThread.Create(AUrl: String);
begin
  inherited Create(true);   // true = suspended
  FreeOnTerminate := true;  // The thread destroys itself when it is finished
  FUrl := AUrl;
  FPHTTPClient := TFPHTTPClient.Create(nil);
  FPHTTPClient.AllowRedirect := true;
end;

destructor TDownloadThread.Destroy;
begin
  FPHTTPClient.Free;
  inherited;
end;

{ This procedure is called from the main thread. You could access GUI elements
  here, but there is greater flexibility if you hook in by means of an event. }
procedure TDownloadThread.DoComplete;
begin
  if Assigned(FOnDownloadComplete) then
    FOnDownloadComplete(FString);
end;

{ This is the worker part of the read, the download. Don't access any GUI element
  here! }
procedure TDownloadThread.Execute;
begin
  try
    FString := FPHTTPClient.Get(FUrl);
    Synchronize(@DoComplete);
  except
  end;
end;


{ TForm1 }

procedure TForm1.FormShow(Sender: TObject);
begin
  loadgraf();
  Timer1.Enabled := true;
end;

procedure TForm1.DownloadComplete(AJSONString: String);
var
  contador: Integer;
  data, dataitem: TJSONData;
  O, H, L, C: Double;
  start, finish, variation: Double;
begin
  currentmarket.Clear;
  data := GetJSON(AJSONString);
  try
    for contador := 0 to data.Count-1 do
    begin
      dataItem := TJSONObject(data.Items[contador]);
      O := dataItem.Items[1].AsFloat;
      H := dataItem.Items[2].AsFloat;
      L := dataItem.Items[3].AsFloat;
      C := dataItem.Items[4].AsFloat;
      if contador = 0 then start := O;
      if contador = data.Count-1 then finish := C;
      currentmarket.AddXOHLC(contador, O, H, L, C);
    end;
    variation := finish / start * 100 - 100;
    Label1.Caption := FormatFloat('0.00%', variation);
  finally
    data.Free;
  end;
end;

procedure TForm1.LoadGraf;
begin
  FDownloadThread := TDownloadThread.Create('https://api.binance.com/api/v1/klines?symbol=TRXBTC&interval=1m&limit=50');
  FDownloadThread.OnDownLoadComplete := @DownloadComplete;
  FDownloadThread.OnTerminate := @ThreadTerminateHandler;
  FDownloadThread.Start;
  // No thread.Free needed because the thread will destroy itself on termination.
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  // Avoid restarting a download while the form is destroyed
  Timer1.Enabled := false;

  // Wait until current download thread is finished.
  if FDownloadThread <> nil then
    FDownloadThread.WaitFor;
end;

procedure TForm1.ThreadTerminateHandler(Sender: TObject);
begin
  // Download is complete. The thread has been destroyed and must be set to nil
  FDownloadThread := nil;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  // Suspend the timer while processing a download
  Timer1.Enabled := false;
  LoadGraf;
  // Restart the timer.
  Timer1.Enabled := true;
end;

end.

